/** This is a linear (fixed effects) model but without the origin and  
    added dispersion components to weight for number of observations nobs 
    in each datapoint **/

data {
  int N;   //number of datapoints
  int K;   //number of predictors for shrinkage
  int M;   //total num predictors, with confounders
  vector<lower=1>[N] nobs;  //number of observations within datapoints
                            //(coded as vector to avoid crash in rmarkdown)
  vector[N]    y;
  matrix[N,M]  preds;  //first predictors are shrunk

  //informative priors for residuals
  real<lower=0> logsigmaloc;
  real<lower=0> logsigmascale;
}
parameters {
  vector[M] beta;  //flat prior
  
  //residual variance model
  real logsigma_within;  //variance within datapoints
  real logsigma_betw;    //variance between datapoints
}
transformed parameters {
  //residual variance model
  real sigma_within;   //within
  real sigma_betw;     //between
  vector[N] disp;     //overall dispersion within + between
  real mndisp;
  
  //predictors
  vector[N] f;     //predictor
  vector[N] fadjust;
  
  //computations residual variance model
  sigma_within = exp(logsigma_within);
  sigma_betw   = exp(logsigma_betw);
  disp   = sqrt(sigma_within^2 ./ nobs + sigma_betw^2);
  mndisp = sqrt(mean(sigma_within^2 ./ nobs + sigma_betw^2));
  
  //linear predictor
  f = preds * beta;
  fadjust = preds[1:N,1:K] * beta[1:K];
}
model {
  //model of the residual component variance
  logsigma_within  ~ normal(logsigmaloc, logsigmascale);
  logsigma_betw    ~ normal(logsigmaloc, logsigmascale);

  //likelihood
  y ~ normal(f, disp);
}
