/** This is the horseshoe model (no regulrization) but without the origin and 
    added dispersion components to weight for number of observations nobs in 
    each datapoint **/

data {
  int N;   //number of datapoints
  int K;   //number of shrunken predictors
  int M;   //total num predictors
  vector<lower=1>[N] nobs;  //number of observations within datapoints
                            //(coded as vector to avoid crash in rmarkdown)
  vector[N]    y;
  matrix[N,M]  preds;  //hs shrunk first
  
  real<lower=0> beta_scale; //prior scale non-shrunken coefs

  real<lower=0> scale_global;
  real<lower=0> scale_local;
  real<lower=1> nu_global;
  real<lower=1> nu_local;

  //informative priors for residuals
  real<lower=0> logsigmaloc;
  real<lower=0> logsigmascale;
}
parameters {
  //residual variance model
  vector[K] z;
  vector[M-K] b;
  real logsigma_within;  //variance within datapoints
  real logsigma_betw;    //variance between datapoints

  //hs ridge model
  real<lower=0> tau;
  vector<lower=0>[K] lambda;
}
transformed parameters {
  //residual variance model
  real sigma_within;    //within
  real sigma_betw;      //between
  vector[N] disp;       //overall dispersion within + between
  real mndisp;

  //predictors
  vector[M] beta;    //regression coefficients
  vector[N] f;       //linear predictor
  vector[N] fadjust; //predicted adjustment
  
  //computations residual variance model
  sigma_within = exp(logsigma_within);
  sigma_betw   = exp(logsigma_betw);
  disp   = sqrt(sigma_within^2 ./ nobs + sigma_betw^2);
  mndisp = sqrt(mean(sigma_within^2 ./ nobs + sigma_betw^2));
  
  //hs ridge
  for (i in 1:M) {
    if (i <= K)
      beta[i] = z[i] * lambda[i] * tau;
    else 
      beta[i] = b[i-K];
  }
  f = preds * beta;
  fadjust = preds[1:N,1:K] * beta[1:K];
}
model {
  //linear model
  z  ~ normal(0, 1);
  for (i in 1:M-K)  b[i] ~ student_t(1, 0, beta_scale);
  
  //model of the residual component variance
  logsigma_within  ~ normal(logsigmaloc, logsigmascale);
  logsigma_betw    ~ normal(logsigmaloc, logsigmascale);

  // half -t priors for lambdas and tau , and inverse - gamma for c^2
  tau ~ student_t(nu_global, 0, scale_global*mndisp);
  lambda ~ student_t(nu_local, 0, scale_local);

  //likelihood
  y ~ normal(f, disp);
}
